import {Box, TextField, Button, Typography, FormControl, InputLabel, Select, MenuItem} from '@mui/material'
import {Create} from '@refinedev/mui'
import {useForm} from '@refinedev/react-hook-form'

import {MEDIA_TYPES} from '../../lib/constants'
import {ALLOWED_IMAGE_TYPES, ALLOWED_VIDEO_TYPES, useMediaFileUpload} from '../../components/media/MediaFormUtils'

export const MediaCreate = () => {
  const {
    saveButtonProps,
    refineCore: {formLoading, onFinish},
    register,
    formState: {errors},
    setValue,
    handleSubmit,
    watch,
    setError,
    clearErrors,
  } = useForm({})

  const selectedMediaType = watch('media_type')

  const {selectedFile, previewUrl, handleFileChange, handleMediaTypeChange} = useMediaFileUpload({
    selectedMediaType,
    setError,
    clearErrors,
    setValue,
  })

  return (
    <Create
      isLoading={formLoading}
      saveButtonProps={{
        ...saveButtonProps,
        disabled: !!errors.file || !selectedFile,
        onClick: handleSubmit((data) => {
          const formData = new FormData()
          formData.append('filename', data.filename)
          formData.append('type', String(data.media_type))
          if (data.file) {
            formData.append('file', data.file)
          }
          onFinish(formData)
        }),
      }}
    >
      <Box component="form" sx={{display: 'flex', flexDirection: 'column'}} autoComplete="off">
        <Box display="flex" flexDirection="column-reverse" alignItems="center" gap={6}>
          <Box display="flex" flexDirection="column" alignItems="center" gap={2}>
            <Button variant="contained" component="label" disabled={!selectedMediaType}>
              {selectedFile ? 'Изменить файл' : 'Загрузить файл'}
              <input type="file" hidden onChange={handleFileChange} accept={selectedMediaType === 1 ? ALLOWED_IMAGE_TYPES.join(',') : ALLOWED_VIDEO_TYPES.join(',')} />
            </Button>

            {selectedFile && (
              <Typography variant="body2" color="text.secondary">
                {selectedFile.name}
              </Typography>
            )}

            {errors.file && (
              <Typography variant="caption" color="error">
                {(errors as any)?.file?.message}
              </Typography>
            )}
          </Box>

          {previewUrl && (
            <Box mt={2} display="flex" justifyContent="center">
              <img src={previewUrl} alt="Preview" style={{maxWidth: '200px', borderRadius: 8}} />
            </Box>
          )}
        </Box>

        <FormControl fullWidth margin="normal" error={!!errors.media_type}>
          <InputLabel id="media-type-label">Тип</InputLabel>
          <Select
            labelId="media-type-label"
            label="Тип"
            {...register('media_type', {
              required: 'Это поле является обязательным',
              valueAsNumber: true,
            })}
            onChange={handleMediaTypeChange}
          >
            {MEDIA_TYPES.map((type) => (
              <MenuItem key={type.value} value={type.value}>
                {type.label}
              </MenuItem>
            ))}
          </Select>
          {errors.media_type && (
            <Typography variant="caption" color="error">
              {(errors as any)?.media_type?.message}
            </Typography>
          )}
        </FormControl>

        <TextField
          {...register('filename', {
            required: 'Это поле является обязательным',
          })}
          error={!!(errors as any)?.filename}
          helperText={(errors as any)?.filename?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{shrink: true}}
          type="text"
          label="Название"
          name="filename"
        />
      </Box>
    </Create>
  )
}
