import {Box, TextField, Button, Typography, FormControl, InputLabel, Select, MenuItem} from '@mui/material'
import {Edit} from '@refinedev/mui'
import {useForm} from '@refinedev/react-hook-form'
import {useEffect} from 'react'
import {useShow} from '@refinedev/core'

import {MEDIA_TYPES} from '../../lib/constants'
import {ALLOWED_IMAGE_TYPES, ALLOWED_VIDEO_TYPES, useMediaFileUpload} from '../../components/media/MediaFormUtils'

type MediaFormValues = {
  filename: string
  media_type: number
  file?: File
}

export const MediaEdit = () => {
  const {
    saveButtonProps,
    refineCore: {onFinish},
    register,
    formState: {errors},
    setValue,
    handleSubmit,
    watch,
    setError,
    clearErrors,
  } = useForm<MediaFormValues>({
    defaultValues: {
      filename: '',
      media_type: '',
      file: undefined,
    },
  })

  const {query} = useShow()
  const {data} = query
  const record = data?.data

  const selectedMediaType = watch('media_type')

  const {selectedFile, previewUrl, setPreviewUrl, handleFileChange, handleMediaTypeChange} = useMediaFileUpload({
    selectedMediaType,
    setError,
    clearErrors,
    setValue,
  })

  useEffect(() => {
    if (record?.id) {
      setPreviewUrl(`https://wn.krbl.ru/media/${record.id}/download`)
      setValue('filename', record?.filename || '')
      setValue('media_type', record?.media_type)
    }
  }, [record, setValue, setPreviewUrl])

  return (
    <Edit
      saveButtonProps={{
        ...saveButtonProps,
        disabled: !!errors.file,
        onClick: handleSubmit((data) => {
          const formData = {
            filename: data.filename,
            type: Number(data.media_type),
          }
          onFinish(formData)
        }),
      }}
    >
      <Box component="form" sx={{display: 'flex', flexDirection: 'column'}} autoComplete="off">
        <Box display="flex" flexDirection="column-reverse" alignItems="center" gap={6}>
          <Box display="flex" flexDirection="column" alignItems="center" gap={2}>
            <Button variant="contained" component="label" disabled={!selectedMediaType}>
              {selectedFile ? 'Изменить файл' : 'Загрузить файл'}
              <input type="file" hidden onChange={handleFileChange} accept={selectedMediaType === 1 ? ALLOWED_IMAGE_TYPES.join(',') : ALLOWED_VIDEO_TYPES.join(',')} />
            </Button>

            {selectedFile && (
              <Typography variant="body2" color="text.secondary">
                {selectedFile.name}
              </Typography>
            )}

            {errors.file && (
              <Typography variant="caption" color="error">
                {(errors as any)?.file?.message}
              </Typography>
            )}
          </Box>

          {previewUrl && (
            <Box mt={2} display="flex" justifyContent="center">
              <img src={previewUrl} alt="Preview" style={{maxWidth: '200px', borderRadius: 8}} />
            </Box>
          )}
        </Box>

        <FormControl fullWidth margin="normal" error={!!errors.media_type}>
          <InputLabel id="media-type-label">Тип</InputLabel>
          <Select
            labelId="media-type-label"
            label="Тип"
            {...register('media_type', {
              required: 'Это поле является обязательным',
              valueAsNumber: true,
            })}
            onChange={handleMediaTypeChange}
          >
            {MEDIA_TYPES.map((type) => (
              <MenuItem key={type.value} value={type.value}>
                {type.label}
              </MenuItem>
            ))}
          </Select>
          {errors.media_type && (
            <Typography variant="caption" color="error">
              {(errors as any)?.media_type?.message}
            </Typography>
          )}
        </FormControl>

        <TextField
          {...register('filename', {
            required: 'Это поле является обязательным',
          })}
          error={!!(errors as any)?.filename}
          helperText={(errors as any)?.filename?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{shrink: true}}
          type="text"
          label="Название"
          name="filename"
        />
      </Box>
    </Edit>
  )
}
